package article_test

import (
	"context"
	"fmt"

	extctx "golang.org/x/net/context"
)

type Foo struct {
	Name string
}

type Bar Foo

type Baz = Foo

func ExampleAlias() {
	fmt.Printf("%T\n", Foo{})
	fmt.Printf("%T\n", Bar{})
	fmt.Printf("%T\n", Baz{})
	// OUTPUT:
	// article_test.Foo
	// article_test.Bar
	// article_test.Foo
}

func HandleBuiltin(_ context.Context) {}
func HandleExternal(_ extctx.Context) {}

func ExampleCompatibility() {
	ctx1 := context.Background()
	ctx2 := extctx.Background()

	HandleBuiltin(ctx1)
	HandleExternal(ctx2)
	HandleBuiltin(ctx2)
	HandleExternal(ctx1)
}

func ExampleIncompatibility() {
	var h func(context.Context)

	h = HandleBuiltin  // OK
	h = HandleExternal // NOT OK (for < go1.9, and old golang.org/x/net/context)
	_ = h
}

func (f Foo) Baz() {
}

// Baz = Fooなのでこれは同じメソッドを宣言したことになり、コンパイルできない
// func (b Baz) Baz() {
// }

type intList = []int

// intList = []int で、配列やint型にはメソッドを追加できないので、
// コンパイルできない
// func (l intList) Len() int {
//	return len(l)
// }
